{{!
SPDX-FileCopyrightText: 2020 Kitsune Ral <Kitsune-Ral@users.sf.net>
SPDX-License-Identifier: LGPL-2.1-or-later
}}{{>preamble}}
#pragma once

#include <Quotient/converters.h>
{{#imports}}
#include {{_}}{{/imports}}

namespace Quotient {
{{>define_models}}
} // namespace Quotient
